package com.kundun.graph.controller;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.kundun.graph.io.Writer;
import com.kundun.graph.io.WriterShellGraph;
import com.kundun.graph.main.Config;
import com.kundun.graph.main.GraphRepository;
import com.kundun.graph.model.Graph;
import com.kundun.graph.parser.MainParser;
import com.kundun.graph.shell.Shell;

public class Display implements Operation {
	
	private final static Logger LOGGER = Logger.getLogger(Display.class.getName());

	private String[] input;
	
	/**
	 * Manage display operation
	 * 
	 * @param input
	 */
	public Display(String[] input) {
		this.input = input;
	}
	
	/**
	 * Display the details of graph with ID = id
	 * 
	 * @param id
	 */
	private static void displayTheGraph(String id) {
		Graph graphToDisplay = Graph.getGraphById(id);
		ArrayList<HashMap<String, String>> hshMpToDisplay = new ArrayList<HashMap<String, String>>();
		
		// Check if the graph exist
		if (graphToDisplay == null) {
			Shell.println("Graph with ID '" + id + "' does not exist!");
			graphToDisplay = null;
			LOGGER.info(Config.ln() + "Graph with ID " + id + " does not exist.");
			LOGGER.debug(Config.ln() + "Graph set to null");
		} else {
			// Display the graph if found
			LOGGER.info(Config.ln() + "Graph with ID " + id + 
					" is being prepared to be displayed");
			// Parse the graph
			hshMpToDisplay = MainParser.objectToHashMap(graphToDisplay);
			
			// Write the graph to console
			Writer writer = new WriterShellGraph();
			try {
				writer.write(hshMpToDisplay);
			} catch (Exception e) {
				LOGGER.error(Config.ln() + "Error parsing graph: ");
				LOGGER.error(e.getMessage());
			}
		}
	}
		
	/**
	 * Display the list of all graphs (its IDs)
	 * 
	 */
	private static void displayAll() {
		int i = 1; // display list counter
		
		if(!GraphRepository.globalGraphList.isEmpty()) {
			for(Graph g : GraphRepository.globalGraphList) {
				Shell.println(i + ": '" + g.getId() + "'");
				i++;
			}
		} else {
			Shell.println("No graphs on list");
		}
	}

	@Override
	public void doOperation() {
		if (input.length < 2) {
			Shell.invalidSyntax();
		} else {
			switch (input[1].toLowerCase()) {
			case ALL:
				displayAll();
				break;
				
			default:
				displayTheGraph(input[1]);
			}
		}
	}
}
