package com.kundun.graph.controller;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import com.kundun.graph.main.Config;
import com.kundun.graph.main.GraphRepository;
import com.kundun.graph.model.Edge;
import com.kundun.graph.model.Graph;
import com.kundun.graph.model.Node;
import com.kundun.graph.shell.Shell;

public class Remove implements Operation {
	
	private final static Logger LOGGER = Logger.getLogger(Remove.class.getName());

	private String[] input;
	
	/**
	 * Manage remove operation
	 * 
	 * @param input
	 */
	public Remove(String[] input) {
		this.input = input;
	}
	

	@Override
	public void doOperation() {
		String graphId, nodeId, edgeId;
		
		if (input.length < 3) {
			Shell.invalidSyntax();
		} else {
			switch (input[1].toLowerCase()) {
			case GRAPH:
				if (input.length < 3) {
					Shell.invalidSyntax();
					break;
				}
	
				graphId = input[2].trim();
				removeGraph(graphId);
				break;
	
			case NODE:
				if (input.length < 4) {
					Shell.invalidSyntax();
					break;
				}
	
				graphId = input[2].trim();
				nodeId = input[3].trim();
				removeNode(graphId, nodeId);
				break;
	
			case EDGE:
				if (input.length < 4) {
					Shell.invalidSyntax();
					break;
				}
	
				graphId = input[2].trim();
				edgeId = input[3].trim();
				removeEdge(graphId, edgeId);
				break;
	
			default:
				Shell.invalidSyntax();
				break;
			}
		}
	}
	
	/**
	 * Remove the graph with given ID
	 * 
	 * @param graphId
	 */
	private void removeGraph(String graphId) {
		Graph graph = Graph.getGraphById(graphId);
		// Check if the graph we want to add the node exist
		if (graph == null) {
			Shell.println("Graph with ID: '" + graphId + "' does not exist!");
		} else {
			try {
				GraphRepository.globalGraphList.remove(graph);
				Shell.println("Graph '" + graphId + "' removed.");
				LOGGER.info(Config.ln() + "Graph '" + graphId + "' removed.");
			} catch (Exception e) {
				LOGGER.error(Config.ln() + "Error removing graph: ");
				LOGGER.error(e.getMessage());
			}
		}
	}
	
	private void removeNode(String graphId, String nodeId) {
		Graph graph = Graph.getGraphById(graphId);
		// Check if the graph we want to add the node exist
		if (graph == null) {
			Shell.println("Graph with ID: '" + graphId + "' does not exist!");
		} else {
			Node node = graph.getNodeById(nodeId);
			if (node == null) {
				Shell.println("Node with ID: '" + nodeId + "' does not exist " +
						" in graph '" + graphId + "'.");
			} else {
				try {
					ArrayList<Edge> edgesToRemove = new ArrayList<Edge>(); 
					for (Edge e : graph.getEdges()) { // also remove all edges going to that node
						if ( e.getNodeFrom().getId().equals(nodeId) || e.getNodeTo().getId().equals(nodeId)) {
							edgesToRemove.add(e);
							Shell.println("Due to removal of node '" + nodeId + "' edge '" + e.getId() +
									"' is also being removed");
							LOGGER.info(Config.ln() + "Due to removal of node '" + nodeId + "' edge '" + e.getId() +
							"' is also being removed");
						}
					}
					graph.removeEdge(graphId, edgesToRemove);
					graph.removeNode(node);
					Shell.println("Node '" + nodeId + "' removed.");
					LOGGER.info(Config.ln() + "Node '" + nodeId + "' removed.");
				} catch (Exception e) {
					LOGGER.error(Config.ln() + "Error removing node: ");
					LOGGER.error(e.getMessage());
				}
			}
		}
	}
	
	/**
	 * Remove an edge from the graph
	 * 
	 * @param graphId
	 * @param edgeId
	 */
	private void removeEdge(String graphId, String edgeId) {
		Graph graph = Graph.getGraphById(graphId);
		// Check if the graph we want to add the node exist
		if (graph == null) {
			Shell.println("Graph with ID: '" + graphId + "' does not exist!");
		} else {
			Edge edge = graph.getEdgeById(edgeId);
			if (edge == null) {
				Shell.println("Edge with ID: '" + edgeId + "' does not exist " +
						" in graph '" + graphId + "'.");
			} else {
				try {
					graph.removeEdge(edge);
					Shell.println("Edge '" + edgeId + "' removed.");
					LOGGER.info(Config.ln() + "Edge '" + edgeId + "' removed.");
				} catch (Exception e) {
					LOGGER.error(Config.ln() + "Error removing edge: ");
					LOGGER.error(e.getMessage());
				}
			}
		}
	}
}
