package com.kundun.graph.controller;

import java.io.IOException;

import org.apache.log4j.Logger;

import com.kundun.graph.io.WriterFile;
import com.kundun.graph.main.Config;
import com.kundun.graph.model.Graph;
import com.kundun.graph.parser.MainParser;
import com.kundun.graph.shell.Shell;

public class Save implements Operation {

	private final static Logger LOGGER = Logger.getLogger(Shell.class.getName());
	
	private String[] input;
	
	/**
	 * Manage save to file operation
	 * 
	 * @param input
	 */
	public Save(String[] input) {
		this.input = input;
	}
	
	@Override
	public void doOperation() {
		if(input.length < 5) {
			Shell.invalidSyntax();
		} else {
			switch (input[3].toLowerCase()) {
			case FILE:
				saveGraph(input);
				break;
				
			default:
				Shell.invalidSyntax(); break;
			}
		}
	}
	
	/**
	 * Operation of saving graph into a file.
	 * Bases on provided ID and path.
	 * 
	 * @param input
	 */
	private void saveGraph(String[] input) {
		String graphId = input[2];
		String filePath = input[4];
		Graph graph = Graph.getGraphById(graphId);
		
		try {
			WriterFile writer = new WriterFile(filePath);
			writer.write(MainParser.objectToHashMap(graph));
			Shell.println("Graph: '" + graphId + "' saved to '" + filePath + "'");
			LOGGER.info(Config.ln() + "Graph: " + graphId + " saved to " + filePath);
			
			// Change save status of graph
			graph.save();
		} catch (IOException e) {
			LOGGER.error(Config.ln() + "Cannot write to file.");
			LOGGER.error(e.getMessage());
		}
	}
}
