package com.kundun.graph.io;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

import org.apache.log4j.Logger;

import com.kundun.graph.main.Config;
import com.kundun.graph.shell.Shell;

public class ReaderFile extends Reader {
	
	private String fEncoding = "UTF-8";
	
	private String fileName;
	private Scanner fileScanner;
	
	private final static Logger LOGGER = Logger.getLogger(ReaderFile.class.getName());
	
	/**
	 * Pin the data file name 
	 * 
	 * @param fileName
	 * @throws FileNotFoundException
	 */
	public ReaderFile(String fileName) throws FileNotFoundException {
		this.fileName = fileName;  
	}
	
	
	/**
	 * Parse file to an array
	 * 
	 * @param fileName
	 * @throws FileNotFoundException
	 * 
	 * @see com.kundun.graph.io.Reader#read()
	 */
	@Override
	public ArrayList<HashMap<String,String>> read() throws FileNotFoundException { 
		ArrayList<HashMap<String,String>> parsingResult = new ArrayList<HashMap<String,String>>();

		// Read line-by-line and insert parsed rows into HashMap
		LOGGER.info(Config.ln() + "Looking for data file: " + fileName);
		try {
			fileScanner = new Scanner(new FileInputStream(fileName), fEncoding);		

			while (fileScanner.hasNextLine()) {
				String[] splitted = (fileScanner.nextLine()).split(":");
				HashMap<String, String> hshMp = new HashMap<String,String>();
				
				// Check if there's proper number of splitted args - 2 - otherwise it's empty line of start/end of graph
				if (splitted.length == 2) {
					hshMp.put(splitted[0].trim().toLowerCase(), splitted[1].trim());
					parsingResult.add(hshMp);
				}
			}
		} catch (FileNotFoundException e) {
			LOGGER.error(Config.ln() + "File " + fileName + " not found");
			LOGGER.error(e.getMessage());
			Shell.println("File '" + fileName + "' not found");
			return null;
		}
		
		LOGGER.info(Config.ln() + "Closing file after reading");
		fileScanner.close();
		
		return parsingResult;
	}
}
