package com.kundun.graph.io;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;
import java.io.IOException;

import org.apache.log4j.Logger;

import com.kundun.graph.main.Config;
import com.kundun.graph.model.Graph;
import com.kundun.graph.parser.MainParser;

public class WriterFile extends Writer {
	
	public BufferedWriter destinationFile;
	
	private final static Logger LOGGER = Logger.getLogger(WriterFile.class.getName());
	
	
	/**
	 * Create new file and pin writer to it
	 * 
	 * @param fileName
	 * @throws IOException
	 */
	public WriterFile(String fileName) throws IOException {
		LOGGER.info(Config.ln() + "Creating new file: " + fileName);
		this.destinationFile = new BufferedWriter(new FileWriter(fileName));
	}
	

	/**
	 * Parse from Array to a file
	 * 
	 * @param graphElement
	 * 
	 * @see com.kundun.graph.io.Writer#write(java.util.ArrayList)
	 */
	@Override
	public void write(ArrayList<HashMap<String,String>> dataToParse) throws IOException {
			
			try {
				LOGGER.info(Config.ln() + "Writing hashmap to file");
				this.destinationFile.write("{\n");
				for (HashMap<String, String> map : dataToParse) {
					for (Entry<String, String> entry : map.entrySet()) {
				         
						this.destinationFile.write(entry.getKey() + GRAMMAR_SEPARATOR + entry.getValue() + "\r\n");
				    }
				}
				this.destinationFile.write("}");
			} finally {
				LOGGER.info(Config.ln() + "Closing file after writing");
				this.destinationFile.close();
			}
	}
	
	/**
	 * Parse from an object to text file
	 * 
	 * @param fileOut
	 * @param graph
	 * @throws IOException
	 */
	public static void parseOut(WriterFile fileOut, Graph graph) 
				throws IOException {

		LOGGER.info(Config.ln() + "Start parsing to file...");
		
		ArrayList<HashMap<String,String>> hashmapOut = new ArrayList<HashMap<String,String>>();
		
		// Now parse out, to another file
		hashmapOut = MainParser.objectToHashMap(graph);
		fileOut.write(hashmapOut);
	}
}
