package com.kundun.graph.main;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import org.apache.log4j.PropertyConfigurator;

public class Config {
	
	private static final Config instance = new Config();
	
    /**
     * Let's do it in a singleton-way
     */
    private Config() {}
    public static Config getInstance() {
        return instance;
    }
	
    
	/**
	 * Initialize logger for application
	 * 
	 * @throws IOException
	 */
	public static void initLogger() throws IOException {
    	String log4jConfigFile = "src" + File.separator + "log4j.properties";
    	File logfile = new File(log4jConfigFile);

    	if (!logfile.canRead()) {
    		System.err.println("log4j.properties does not exist:" + logfile.getAbsolutePath());
    		return;
    	} else {
    		FileInputStream in = new FileInputStream(log4jConfigFile);
    		Properties prop = new Properties();
    		prop.load(in);
    		in.close();
    		prop.setProperty("log4j.appender.A1.File", "log.txt");
    		PropertyConfigurator.configure(prop);
    	}
	}

	/**
	 * Get the current line number
	 * 
	 * @return int
	 */
	private static int getLineNumber() {
	    return Thread.currentThread().getStackTrace()[3].getLineNumber();
	}
	

	/**
	 * Print logger line of code
	 * 
	 * @return String
	 */
	public static String ln() {
		return "@" + getLineNumber() + " ";
	}
}
