package com.kundun.graph.model;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import com.kundun.graph.main.Config;
import com.kundun.graph.main.GraphRepository;

public abstract class Graph {
	
	private String id;
	private String label;
	private ArrayList<Node> nodes = new ArrayList<Node>();
	private ArrayList<Edge> edges = new ArrayList<Edge>();
	protected Boolean saved;
	
	private final static Logger LOGGER = Logger.getLogger(Graph.class.getName());

	protected abstract Graph createGraph(String id, String label);
	protected abstract Graph createGraph(String id, String label, ArrayList<Node> nodes, ArrayList<Edge> edges);

	public String getId() {
		return id;
	}
	public void setId(String id) {
		this.id = id;
	}
		
	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public ArrayList<Node> getNodes() {
		return nodes;
	}

	public void setNodes(ArrayList<Node> nodes) {
		this.nodes = nodes;
	}

	public ArrayList<Edge> getEdges() {
		return edges;
	}

	public void setEdges(ArrayList<Edge> edges) {
		this.edges = edges;
	}
	
	public void addNode(Node node) {
		this.nodes.add(node);
	}
	
	public void removeNode(Node node) {
		this.nodes.remove(node);
	}
	
	public void addEdge(Edge edge) {
		this.edges.add(edge);
	}
	
	public void removeEdge(Edge edge) {
		this.edges.remove(edge);
	}
	
	public void removeEdge(String graphId, ArrayList<Edge> edgesToRemove) {
		this.edges.removeAll(edgesToRemove);
	}
	
	/**
	 * Change graph status to 'saved'
	 */
	public void save() {
		this.saved = true;
	}
	
	/**
	 * Change graph status to 'unsaved'
	 */
	public void unsave() {
		this.saved = false;
	}
	
	/**
	 * Get save status of the graph
	 * 
	 * @return boolean
	 */
	public Boolean isSaved() {
		return this.saved;
	}
	
	/**
	 * Go through global graph list and fetch one basing on ID
	 * 
	 * @param id
	 * @return graph
	 */
	public static Graph getGraphById(String id) {
		for(Graph g : GraphRepository.globalGraphList) {
			if(g.getId().equals(id)) {
				return g;
			}
		}
		LOGGER.info(Config.ln() + "Graph with ID: " + id + " is unique");
		return null;
	}
	
	/**
	 * Get node with ID 'id' from all graph nodes
	 * 
	 * @param id
	 * @return Node
	 */
	public Node getNodeById(String id) {
		for(Node n : this.nodes) {
			if ((n.getId()).equals(id)) 
				return n;
		}
		LOGGER.info(Config.ln() + "Node with ID: " + id + " not found in graph '" + this.getId() + "'!");
		return null;
	}
	
	/**
	 * Get edge with ID 'id' from all graph edges
	 * 
	 * @param id
	 * @return Edge
	 */
	public Edge getEdgeById(String id) {
		for(Edge e : this.edges) {
			if ((e.getId()).equals(id)) 
				return e;
		}
		LOGGER.info(Config.ln() + "Edge with ID: " + id + " not found in graph '" + this.getId() + "'!");
		return null;
	}
}
