package com.kundun.graph.model;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import com.kundun.graph.main.Config;

public class Node {
	
	private String id;
	private String label;
	
	private final static Logger LOGGER = Logger.getLogger(Node.class.getName());
	
	public Node(String id, String label) {
		
		LOGGER.info(Config.ln() + "Creating new node with ID: " + id);
		
		this.setId(id);
		this.setLabel(label);
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}
	
	
	/**
	 * Fetch particular Node from ArrayList using its ID
	 * 
	 * @param list
	 * @param id
	 * @return Node
	 */
	public static Node getNodeById(ArrayList<Node> list, String id) {
		
		for(Node n : list) {
			if ((n.getId()).equals(id)) 
				return n;
		}
		LOGGER.info(Config.ln() + "Node with ID: " + id + " not found!");
		return null;
	}
	
	/**
	 * Check uniqueness of Node ID
	 * 
	 * @param list
	 * @param id
	 * @param label
	 * @return
	 * @throws Exception
	 */
	public static Node checkAndCreateUniqueNode(ArrayList<Node> list, String id, String label) throws Exception {
		
		for(Node n : list) {
			if ((n.getId()).equals(id)) {
				LOGGER.info(Config.ln() + "Node with ID: " + id + " is NOT unique - creating failed.");
				throw new Exception();
					
			}
		}
		LOGGER.info(Config.ln() + "Node with ID: " + id + " is unique - creating succesfull.");
		return new Node(id, label);
	}
}
