package com.kundun.graph.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import com.kundun.graph.controller.Add;
import com.kundun.graph.controller.Remove;
import com.kundun.graph.main.Config;
import com.kundun.graph.model.Edge;
import com.kundun.graph.model.Graph;
import com.kundun.graph.model.Node;

public class MainParser {
	
	private final static Logger LOGGER = Logger.getLogger(MainParser.class.getName());
	public static final String SEPARATOR = ",";

	/**
	 * Parse from pre-parsed file to a Graph object
	 * 
	 * @param data
	 * @return Graph
	 * @throws Exception
	 */
	public static void hashMapToObject (ArrayList<HashMap<String,String>> data) throws Exception {
		
		LOGGER.info(Config.ln() + "Fetching graph object from a HashMap");

		String graphId = new String();
		String split[];
		
		// Iterate through HashMap and create objects to finally create Graph object
		for (HashMap<String, String> map : data) {
			for (Entry<String, String> entry : map.entrySet()) {
				// Depending on object type...
				switch(entry.getKey().toLowerCase()) {
					
					// Initiate creating graph
					case "graph":
						LOGGER.info(Config.ln() + "Initiate creating graph");
						
						split = entry.getValue().split(SEPARATOR);
						
						graphId = split[0].trim();
						
						String[] inputG = {"add", "-g", graphId, split[1].trim()};
						new Add(inputG).doOperation();
						
						LOGGER.info(Config.ln() + "Set graph id to: " + graphId + 
								" and label to: " + split[1].trim());
						break;
					
					// Add nodes
					case "node":
						LOGGER.info(Config.ln() + "Initiate creating node");
						
						split = entry.getValue().split(SEPARATOR);
						
						String[] inputN = {"add", "-n", graphId, split[0].trim(), split[1].trim()};
						new Add(inputN).doOperation();
						
						LOGGER.info(Config.ln() + "Set node ID to " + split[0].trim());
						break;
					
					// "Draw" edges
					case "edge":
						LOGGER.info(Config.ln() + "Initiate creating edge");
						
						split = entry.getValue().split(SEPARATOR);
						
						String[] inputE = {"add", "-e", graphId, split[0].trim(), 
								split[1].trim(), split[2].trim(), split[3].trim()};
						new Add(inputE).doOperation();
						
						LOGGER.info(Config.ln() + "Set edge between nodes " + split[1] + " and " + split[2]);
						break;
						

					case "add_node":
						LOGGER.info(Config.ln() + "Initiate creating node from command");
						
						split = entry.getValue().split(SEPARATOR);
						graphId = split[0].trim();
						
						String[] inputAn = {"add", "-n", graphId, split[1].trim(), split[2].trim()};
						new Add(inputAn).doOperation();
						
						LOGGER.info(Config.ln() + "Set node ID to " + split[1].trim() + 
								" in graph '" + graphId + "'.");
						break;
						
					case "add_edge":
						LOGGER.info(Config.ln() + "Initiate creating edge from command");
						
						split = entry.getValue().split(SEPARATOR);
						graphId = split[0].trim();
						
						String[] inputAe = {"add", "-e", graphId, split[1].trim(), 
								split[2].trim(), split[3].trim(), split[4].trim()};
						new Add(inputAe).doOperation();
						
						LOGGER.info(Config.ln() + "Set edge between nodes " + split[2] + " and " + split[3] + 
								" in graph '" + graphId + "'.");
						break;
						
					case "remove_graph":
						LOGGER.info(Config.ln() + "Initiate removing graph from command");
						
						split = entry.getValue().split(SEPARATOR);
						graphId = split[0].trim();
						
						String[] inputRg = {"rm", "-g", graphId};
						new Remove(inputRg).doOperation();
						
						LOGGER.info(Config.ln() + "Graph '" + graphId + "' removed with a file command.");
						break;
						
					case "remove_node":
						LOGGER.info(Config.ln() + "Initiate removing node from command");
						
						split = entry.getValue().split(SEPARATOR);
						graphId = split[0].trim();
						
						String[] inputRn = {"rm", "-n", graphId, split[1].trim()};
						new Remove(inputRn).doOperation();
						
						LOGGER.info(Config.ln() + "Node '" + split[1].trim() + 
								"' removed from graph '" + graphId + "' with a file command.");
						break;
						
					case "remove_edge":
						LOGGER.info(Config.ln() + "Initiate creating edge from command");
						
						split = entry.getValue().split(SEPARATOR);
						graphId = split[0].trim();
						
						String[] inputRe = {"rm", "-e", graphId, split[1].trim()};
						new Remove(inputRe).doOperation();
						
						LOGGER.info(Config.ln() + "Edge '" + split[1].trim() + 
								"' removed from graph '" + graphId + "' with a file command.");
						break;
				}
			}
		}
	}

	
	/**
	 * Parse from a Graph object to a later-to-parse-to-file HashMap object
	 * 
	 * @param graph
	 * @return list of graphElements
	 */
	public static ArrayList<HashMap<String,String>> objectToHashMap (Graph graph) {
		
		LOGGER.info(Config.ln() + "Creating new HashMap from a graph object");
		
		ArrayList<HashMap<String,String>> listOfObjects = new ArrayList<HashMap<String,String>>();
		
		// Fetch nodes and edges from the graph
		ArrayList<Node> nodes = graph.getNodes();
		ArrayList<Edge> edges = graph.getEdges();
		
		// Get graph label
		HashMap<String,String> gHshMp = new HashMap<String,String>();
		gHshMp.put("graph", graph.getId() + SEPARATOR + graph.getLabel());
		
		LOGGER.info(Config.ln() + "Put graph label " + graph.getLabel());
		
		listOfObjects.add(gHshMp);
		
		// Get nodes
		if (!nodes.isEmpty()) {
			for (Node n : nodes) {
				HashMap<String,String> nHshMp = new HashMap<String,String>();
				nHshMp.put("node", n.getId() + SEPARATOR + n.getLabel());
				
				LOGGER.info(Config.ln() + "Put node with ID " + n.getId());
				
				listOfObjects.add(nHshMp);
			}
		}
		
		// Get edges
		if (!edges.isEmpty()) {
			for(Edge e : edges) {
				HashMap<String,String> eHshMp = new HashMap<String,String>();
				eHshMp.put("edge", e.getId() + ", " + e.getNodeFrom().getId() + SEPARATOR + e.getNodeTo().getId() + SEPARATOR + e.getLabel());
				
				LOGGER.info(Config.ln() + "Put edge between " + e.getNodeFrom().getLabel() + " and " +  e.getNodeTo().getLabel());
				
				listOfObjects.add(eHshMp);
			}
		}
		
		return listOfObjects;
	}
}
